@@ -20,8 +20,8 @@ import numpy as np
 
 
 from ..model import Range, Dataset
-from ..utils import geometry, datetime_to_seconds_since_1970
-from ..utils.dates import normalise_dt
+from ..utils import geometry
+from ..utils.dates import normalise_dt, tz_aware
 
 _LOG = logging.getLogger(__name__)
 
@@ -29,6 +29,8 @@ _LOG = logging.getLogger(__name__)
 class GroupBy:
     def __init__(self, group_by_func, dimension, units, sort_key=None, group_key=None):
         """
+        GroupBy Object
+
         :param group_by_func: Dataset -> group identifier
         :param dimension: dimension of the group key
         :param units: units of the group key
@@ -190,9 +192,28 @@ def _extract_time_from_ds(ds: Dataset) -> datetime.datetime:
 
 
 def query_group_by(group_by='time', **kwargs):
-    if not isinstance(group_by, str):
+    """
+    Group by function for loading datasets
+
+    :param group_by: group_by name, supported str are
+    ::
+
+        - time (default)
+        - solar_day, see :func:`datacube.api.query.solar_day`
+
+    or ::
+
+        - :class:`datacube.api.query.GroupBy` object
+
+    :return: :class:`datacube.api.query.GroupBy`
+    :raises LookupError: when group_by string is not a valid dictionary key.
+    """
+    if isinstance(group_by, GroupBy):
         return group_by
 
+    if not isinstance(group_by, str):
+        group_by = None
+
     time_grouper = GroupBy(group_by_func=_extract_time_from_ds,
                            dimension='time',
                            units='seconds since 1970-01-01 00:00:00')
@@ -212,7 +233,9 @@ def query_group_by(group_by='time', **kwargs):
     try:
         return group_by_map[group_by]
     except KeyError:
-        raise LookupError('No group by function for', group_by)
+        raise LookupError(
+            f'No group by function for {group_by}, valid options are: {group_by_map.keys()}',  # pylint: disable=W1655
+        )
 
 
 def _range_to_geopolygon(**kwargs):
@@ -281,12 +304,6 @@ def _values_to_search(**kwargs):
     return search
 
 
-def _datetime_to_timestamp(dt):
-    if not isinstance(dt, datetime.datetime) and not isinstance(dt, datetime.date):
-        dt = _to_datetime(dt)
-    return datetime_to_seconds_since_1970(dt)
-
-
 def _to_datetime(t):
     if isinstance(t, (float, int)):
         t = datetime.datetime.fromtimestamp(t, tz=tz.tzutc())
@@ -299,9 +316,7 @@ def _to_datetime(t):
         except ValueError:
             pass
     elif isinstance(t, datetime.datetime):
-        if t.tzinfo is None:
-            t = t.replace(tzinfo=tz.tzutc())
-        return t
+        return tz_aware(t)
 
     return pandas_to_datetime(t, utc=True, infer_datetime_format=True).to_pydatetime()
 
