@@ -15,17 +15,18 @@ Persistence API implementation for postgres.
 
 import logging
 import uuid  # noqa: F401
+from typing import Iterable, Tuple
 from sqlalchemy import cast
 from sqlalchemy import delete
 from sqlalchemy import select, text, bindparam, and_, or_, func, literal, distinct
 from sqlalchemy.dialects.postgresql import INTERVAL
 from sqlalchemy.dialects.postgresql import JSONB, insert
 from sqlalchemy.exc import IntegrityError
-from typing import Iterable, Tuple
 
 from datacube.index.exceptions import MissingRecordError
 from datacube.index.fields import OrExpression
 from datacube.model import Range
+from datacube.utils.uris import split_uri
 from . import _core
 from . import _dynamic as dynamic
 from ._fields import parse_fields, Expression, PgField, PgExpression  # noqa: F401
@@ -33,6 +34,9 @@ from ._fields import NativeField, DateDocField, SimpleDocField
 from ._schema import DATASET, DATASET_SOURCE, METADATA_TYPE, DATASET_LOCATION, PRODUCT
 from .sql import escape_pg_identifier
 
+PGCODE_FOREIGN_KEY_VIOLATION = '23503'
+_LOG = logging.getLogger(__name__)
+
 
 def _dataset_uri_field(table):
     return table.c.uri_scheme + ':' + table.c.uri_body
@@ -58,23 +62,24 @@ _DATASET_SELECT_FIELDS = (
         ).label('uris')
     ).label('uris')
 )
-
-PGCODE_UNIQUE_CONSTRAINT = '23505'
-PGCODE_FOREIGN_KEY_VIOLATION = '23503'
-
-_LOG = logging.getLogger(__name__)
-
-
-def _split_uri(uri):
-    """
-    Split the scheme and the remainder of the URI.
-
-    """
-    idx = uri.find(':')
-    if idx < 0:
-        raise ValueError("Not a URI")
-
-    return uri[:idx], uri[idx+1:]
+_DATASET_BULK_SELECT_FIELDS = (
+    PRODUCT.c.name,
+    DATASET.c.metadata,
+    # All active URIs, from newest to oldest
+    func.array(
+        select([
+            _dataset_uri_field(SELECTED_DATASET_LOCATION)
+        ]).where(
+            and_(
+                SELECTED_DATASET_LOCATION.c.dataset_ref == DATASET.c.id,
+                SELECTED_DATASET_LOCATION.c.archived == None
+            )
+        ).order_by(
+            SELECTED_DATASET_LOCATION.c.added.desc(),
+            SELECTED_DATASET_LOCATION.c.id.desc()
+        ).label('uris')
+    ).label('uris')
+)
 
 
 def get_native_fields():
@@ -177,19 +182,27 @@ def get_dataset_fields(metadata_type_definition):
 class PostgresDbAPI(object):
     def __init__(self, connection):
         self._connection = connection
+        self._sqla_txn = None
 
     @property
     def in_transaction(self):
         return self._connection.in_transaction()
 
     def begin(self):
-        self._connection.execute(text('BEGIN'))
+        self._connection.execution_options(isolation_level="SERIALIZABLE")
+        self._sqla_txn = self._connection.begin()
 
-    def rollback(self):
-        self._connection.execute(text('ROLLBACK'))
+    def _end_transaction(self):
+        self._sqla_txn = None
+        self._connection.execution_options(isolation_level="AUTOCOMMIT")
 
     def commit(self):
-        self._connection.execute(text('COMMIT'))
+        self._sqla_txn.commit()
+        self._end_transaction()
+
+    def rollback(self):
+        self._sqla_txn.rollback()
+        self._end_transaction()
 
     def execute(self, command):
         return self._connection.execute(command)
@@ -225,6 +238,13 @@ class PostgresDbAPI(object):
         )
         return ret.rowcount > 0
 
+    def insert_dataset_bulk(self, values):
+        requested = len(values)
+        res = self._connection.execute(
+            insert(DATASET), values
+        )
+        return res.rowcount, requested - res.rowcount
+
     def update_dataset(self, metadata_doc, dataset_id, product_id):
         """
         Update dataset
@@ -255,7 +275,7 @@ class PostgresDbAPI(object):
         :rtype bool:
         """
 
-        scheme, body = _split_uri(uri)
+        scheme, body = split_uri(uri)
 
         r = self._connection.execute(
             insert(DATASET_LOCATION).on_conflict_do_nothing(
@@ -268,6 +288,11 @@ class PostgresDbAPI(object):
 
         return r.rowcount > 0
 
+    def insert_dataset_location_bulk(self, values):
+        requested = len(values)
+        res = self._connection.execute(insert(DATASET_LOCATION), values)
+        return res.rowcount, requested - res.rowcount
+
     def contains_dataset(self, dataset_id):
         return bool(
             self._connection.execute(
@@ -290,7 +315,7 @@ class PostgresDbAPI(object):
                 )).fetchall()]
 
     def get_datasets_for_location(self, uri, mode=None):
-        scheme, body = _split_uri(uri)
+        scheme, body = split_uri(uri)
 
         if mode is None:
             mode = 'exact' if body.count('#') > 0 else 'prefix'
@@ -455,6 +480,18 @@ class PostgresDbAPI(object):
             select(_DATASET_SELECT_FIELDS).where(DATASET.c.metadata.contains(metadata))
         ).fetchall()
 
+    def search_products_by_metadata(self, metadata):
+        """
+        Find any products that have the given metadata.
+
+        :type metadata: dict
+        :rtype: dict
+        """
+        # Find any products types whose metadata document contains the passed in metadata
+        return self._connection.execute(
+            PRODUCT.select().where(PRODUCT.c.metadata.contains(metadata))
+        ).fetchall()
+
     @staticmethod
     def _alchemify_expressions(expressions):
         def raw_expr(expression):
@@ -569,6 +606,37 @@ class PostgresDbAPI(object):
                                                   select_fields, with_source_ids, limit)
         return self._connection.execute(select_query)
 
+    def bulk_simple_dataset_search(self, products=None, batch_size=0):
+        """
+        Perform bulk database reads (e.g. for index cloning)
+
+        :param products: Optional iterable of product names.  Only fetch nominated products.
+        :param batch_size: Number of streamed rows to fetch from database at once.
+                           Defaults to zero, which means no streaming.
+                           Note streaming is only supported inside a transaction.
+        :return: Iterable of tuples of:
+                 * Product name
+                 * Dataset metadata document
+                 * array of uris
+        """
+        if batch_size > 0 and not self.in_transaction:
+            raise ValueError("Postgresql bulk reads must occur within a transaction.")
+        if products:
+            query = select(PRODUCT.c.id).select_from(PRODUCT).where(PRODUCT.c.name.in_(products))
+            products = [row[0] for row in self._connection.execute(query)]
+            if not products:
+                return []
+        else:
+            products = None
+        query = select(
+            _DATASET_BULK_SELECT_FIELDS
+        ).select_from(DATASET).join(PRODUCT).where(
+            DATASET.c.archived == None
+        )
+        if products:
+            query = query.where(DATASET.c.dataset_type_ref.in_(products))
+        return self._connection.execution_options(stream_results=True, yield_per=batch_size).execute(query)
+
     @staticmethod
     def search_unique_datasets_query(expressions, select_fields, limit):
         """
@@ -648,8 +716,7 @@ class PostgresDbAPI(object):
 
         return self._connection.execute(select_query)
 
-    def get_duplicates(self, match_fields, expressions):
-        # type: (Tuple[PgField], Tuple[PgExpression]) -> Iterable[tuple]
+    def get_duplicates(self, match_fields: Iterable[PgField], expressions: Iterable[PgExpression]) -> Iterable[Tuple]:
         group_expressions = tuple(f.alchemy_expression for f in match_fields)
 
         select_query = select(
@@ -937,6 +1004,11 @@ class PostgresDbAPI(object):
             PRODUCT.select().order_by(PRODUCT.c.name.asc())
         ).fetchall()
 
+    def get_all_product_docs(self):
+        return self._connection.execute(
+            select(PRODUCT.c.definition)
+        )
+
     def _get_products_for_metadata_type(self, id_):
         return self._connection.execute(
             PRODUCT.select(
@@ -949,6 +1021,19 @@ class PostgresDbAPI(object):
     def get_all_metadata_types(self):
         return self._connection.execute(METADATA_TYPE.select().order_by(METADATA_TYPE.c.name.asc())).fetchall()
 
+    def get_all_metadata_type_docs(self):
+        return self._connection.execute(
+            select(METADATA_TYPE.c.definition)
+        )
+
+    def get_all_metadata_defs(self):
+        return [
+            r[0]
+            for r in self._connection.execute(
+                METADATA_TYPE.select(METADATA_TYPE.c.definition).order_by(METADATA_TYPE.c.name.asc())
+            ).fetchall()
+        ]
+
     def get_locations(self, dataset_id):
         return [
             record[0]
@@ -987,7 +1072,7 @@ class PostgresDbAPI(object):
 
         :returns bool: Was the location deleted?
         """
-        scheme, body = _split_uri(uri)
+        scheme, body = split_uri(uri)
         res = self._connection.execute(
             delete(DATASET_LOCATION).where(
                 and_(
@@ -1000,7 +1085,7 @@ class PostgresDbAPI(object):
         return res.rowcount > 0
 
     def archive_location(self, dataset_id, uri):
-        scheme, body = _split_uri(uri)
+        scheme, body = split_uri(uri)
         res = self._connection.execute(
             DATASET_LOCATION.update().where(
                 and_(
@@ -1016,7 +1101,7 @@ class PostgresDbAPI(object):
         return res.rowcount > 0
 
     def restore_location(self, dataset_id, uri):
-        scheme, body = _split_uri(uri)
+        scheme, body = split_uri(uri)
         res = self._connection.execute(
             DATASET_LOCATION.update().where(
                 and_(
@@ -1060,14 +1145,12 @@ class PostgresDbAPI(object):
             self._connection.execute(sql,
                                      description=description)
 
-    def drop_users(self, users):
-        # type: (Iterable[str]) -> None
+    def drop_users(self, users: Iterable[str]) -> None:
         for username in users:
             sql = text('drop role {username}'.format(username=escape_pg_identifier(self._connection, username)))
             self._connection.execute(sql)
 
-    def grant_role(self, role, users):
-        # type: (str, Iterable[str]) -> None
+    def grant_role(self, role: str, users: Iterable[str]) -> None:
         """
         Grant a role to a user.
         """
