@@ -3,23 +3,27 @@
 # Copyright (c) 2015-2020 ODC Contributors
 # SPDX-License-Identifier: Apache-2.0
 import logging
-import warnings
+from time import monotonic
+from typing import Iterable
 
 from cachetools.func import lru_cache
 
-from datacube.index.abstract import AbstractMetadataTypeResource
+from datacube.index.abstract import AbstractMetadataTypeResource, BatchStatus
+from datacube.index.postgis._transaction import IndexResourceAddIn
 from datacube.model import MetadataType
 from datacube.utils import jsonify_document, changes, _readable_offset
 from datacube.utils.changes import check_doc_unchanged, get_doc_changes
 
 _LOG = logging.getLogger(__name__)
 
-class MetadataTypeResource(AbstractMetadataTypeResource):
-    def __init__(self, db):
+
+class MetadataTypeResource(AbstractMetadataTypeResource, IndexResourceAddIn):
+    def __init__(self, db, index):
         """
         :type db: datacube.drivers.postgis._connections.PostgresDb
         """
         self._db = db
+        self._index = index
 
         self.get_unsafe = lru_cache()(self.get_unsafe)
         self.get_by_name_unsafe = lru_cache()(self.get_by_name_unsafe)
@@ -41,7 +45,6 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
         :param dict definition:
         :rtype: datacube.model.MetadataType
         """
-        MetadataType.validate(definition)
         return self._make(definition)
 
     def add(self, metadata_type, allow_table_lock=False):
@@ -55,25 +58,33 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
         :rtype: datacube.model.MetadataType
         """
         # This column duplication is getting out of hand:
-        MetadataType.validate(metadata_type.definition)
+        MetadataType.validate_eo3(metadata_type.definition)
 
         existing = self.get_by_name(metadata_type.name)
         if existing:
             # They've passed us the same one again. Make sure it matches what is stored.
+            _LOG.warning(f"Metadata Type {metadata_type.name} is already in the database, checking for differences")
             check_doc_unchanged(
                 existing.definition,
                 jsonify_document(metadata_type.definition),
                 'Metadata Type {}'.format(metadata_type.name)
             )
         else:
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 connection.insert_metadata_type(
                     name=metadata_type.name,
                     definition=metadata_type.definition,
-                    concurrently=not allow_table_lock
                 )
         return self.get_by_name(metadata_type.name)
 
+    def _add_batch(self, batch_types: Iterable[MetadataType]) -> BatchStatus:
+        # Add a "batch" of mdts.  Simple loop in a transaction for now.
+        b_started = monotonic()
+        values = [{"name": mdt.name, "definition": mdt.definition} for mdt in batch_types]
+        with self._db_connection() as connection:
+            added, skipped = connection.insert_metadata_bulk(values)
+            return BatchStatus(added, skipped, monotonic() - b_started)
+
     def can_update(self, metadata_type, allow_unsafe_updates=False):
         """
         Check if metadata type can be updated. Return bool,safe_changes,unsafe_changes
@@ -127,7 +138,7 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
         can_update, safe_changes, unsafe_changes = self.can_update(metadata_type, allow_unsafe_updates)
 
         if not safe_changes and not unsafe_changes:
-            _LOG.info("No changes detected for metadata type %s", metadata_type.name)
+            _LOG.warning("No changes detected for metadata type %s", metadata_type.name)
             return self.get_by_name(metadata_type.name)
 
         if not can_update:
@@ -140,11 +151,10 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
 
         _LOG.info("Updating metadata type %s", metadata_type.name)
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             connection.update_metadata_type(
                 name=metadata_type.name,
                 definition=metadata_type.definition,
-                concurrently=not allow_table_lock
             )
 
         self.get_by_name_unsafe.cache_clear()   # type: ignore[attr-defined]
@@ -166,7 +176,7 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
     # This is memoized in the constructor
     # pylint: disable=method-hidden
     def get_unsafe(self, id_):  # type: ignore
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             record = connection.get_metadata_type(id_)
         if record is None:
             raise KeyError('%s is not a valid MetadataType id')
@@ -175,13 +185,13 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
     # This is memoized in the constructor
     # pylint: disable=method-hidden
     def get_by_name_unsafe(self, name):  # type: ignore
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             record = connection.get_metadata_type_by_name(name)
         if not record:
             raise KeyError('%s is not a valid MetadataType name' % name)
         return self._make_from_query_row(record)
 
-    def check_field_indexes(self, allow_table_lock=False, rebuild_all=None,
+    def check_field_indexes(self, allow_table_lock=False,
                             rebuild_views=False, rebuild_indexes=False):
         """
         Create or replace per-field indexes and views.
@@ -191,19 +201,7 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
 
             If false, creation will be slightly slower and cannot be done in a transaction.
         """
-        if rebuild_all is not None:
-            warnings.warn(
-                "The rebuild_all option of check_field_indexes() is deprecated.",
-                "Instead, use rebuild_views=True or rebuild_indexes=True as needed.",
-                DeprecationWarning)
-            rebuild_views = rebuild_indexes = rebuild_all
-
-        with self._db.connect() as connection:
-            connection.check_dynamic_fields(
-                concurrently=not allow_table_lock,
-                rebuild_indexes=rebuild_indexes,
-                rebuild_views=rebuild_views,
-            )
+        pass
 
     def get_all(self):
         """
@@ -211,9 +209,14 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
 
         :rtype: iter[datacube.model.MetadataType]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return self._make_many(connection.get_all_metadata_types())
 
+    def get_all_docs(self):
+        with self._db_connection() as connection:
+            for defn in connection.get_all_metadata_type_defs():
+                yield defn
+
     def _make_many(self, query_rows):
         """
         :rtype: list[datacube.model.MetadataType]
@@ -232,6 +235,7 @@ class MetadataTypeResource(AbstractMetadataTypeResource):
         :param int id_:
         :rtype: datacube.model.MetadataType
         """
+        MetadataType.validate_eo3(definition)
         return MetadataType(
             definition,
             dataset_search_fields=self._db.get_dataset_fields(definition),
