@@ -211,6 +211,11 @@ class MetadataTypeResource(AbstractMetadataTypeResource, IndexResourceAddIn):
         with self._db_connection() as connection:
             return self._make_many(connection.get_all_metadata_types())
 
+    def get_all_docs(self):
+        with self._db_connection() as connection:
+            for row in connection.get_all_metadata_type_docs():
+                yield row[0]
+
     def _make_many(self, query_rows):
         """
         :rtype: list[datacube.model.MetadataType]
