@@ -6,8 +6,6 @@ import csv
 import io
 from typing import Iterable, Tuple, Dict, List
 
-from dateutil import tz
-
 import datacube.scripts.search_tool
 from datacube.model import Product, Dataset
 
@@ -26,13 +24,6 @@ def _load_product_query(
     return products
 
 
-def assume_utc(d):
-    if d.tzinfo is None:
-        return d.replace(tzinfo=tz.tzutc())
-    else:
-        return d.astimezone(tz.tzutc())
-
-
 def _csv_search_raw(args, clirunner):
     # Do a CSV search from the cli, returning output as a string
     result = clirunner(['-f', 'csv'] + list(args), cli_method=datacube.scripts.search_tool.cli, verbose_flag=False)
