@@ -141,6 +141,14 @@ def test_mem_product_resource(mem_index_fresh,
             assert unmatched == {}
         else:
             assert unmatched["platform"] == 'landsat-8'
+    lds = list(mem_index_fresh.index.products.search_by_metadata({"product_family": "ard"}))
+    assert len(lds) == 0
+    lds = list(mem_index_fresh.index.products.search_by_metadata({"odc:product_family": "ard"}))
+    assert len(lds) == 1
+    lds = list(mem_index_fresh.index.products.search_by_metadata({"platform": "landsat-8"}))
+    assert len(lds) == 0
+    lds = list(mem_index_fresh.index.products.search_by_metadata({"eo:platform": "landsat-8"}))
+    assert len(lds) == 1
 
 
 # Hand crafted tests with recent real-world eo3 examples
@@ -186,13 +194,13 @@ def test_mem_ds_lineage(mem_eo3_data):
     derived = list(dc.index.datasets.get_derived(ls8_id))
     assert len(derived) == 1
     assert derived[0].id == wo_id
-    assert "cloud_cover" in dc.index.datasets.get_field_names(ls8_ds.type.name)
+    assert "cloud_cover" in dc.index.datasets.get_field_names(ls8_ds.product.name)
 
 
 def test_mem_ds_search_dups(mem_eo3_data):
     dc, ls8_id, wo_id = mem_eo3_data
     ls8_ds = dc.index.datasets.get(ls8_id)
-    dup_results = dc.index.datasets.search_product_duplicates(ls8_ds.type, "cloud_cover", "dataset_maturity")
+    dup_results = dc.index.datasets.search_product_duplicates(ls8_ds.product, "cloud_cover", "dataset_maturity")
     assert len(dup_results) == 1
     assert dup_results[0][0].cloud_cover == ls8_ds.metadata.cloud_cover
     assert ls8_id in dup_results[0][1]
@@ -429,7 +437,7 @@ def test_mem_ds_search_and_count_by_product(mem_eo3_data):
     assert len(lds) == 2
     for dss, product in lds:
         for ds in dss:
-            assert ds.type.name == product.name
+            assert ds.product.name == product.name
     lds = list(dc.index.datasets.count_by_product(platform='landsat-8'))
     assert len(lds) == 2
     for prod, count in lds:
@@ -473,13 +481,13 @@ def test_mem_ds_search_returning_datasets_light(mem_eo3_data):
 
 def test_mem_ds_search_by_metadata(mem_eo3_data):
     dc, ls8_id, wo_id = mem_eo3_data
-    lds = list(dc.index.datasets.search_by_metadata({"product_family": "ard"}))
+    lds = list(dc.index.datasets.search_by_metadata({"properties": {"product_family": "ard"}}))
     assert len(lds) == 0
-    lds = list(dc.index.datasets.search_by_metadata({"odc:product_family": "ard"}))
+    lds = list(dc.index.datasets.search_by_metadata({"properties": {"odc:product_family": "ard"}}))
     assert len(lds) == 1
-    lds = list(dc.index.datasets.search_by_metadata({"platform": "landsat-8"}))
+    lds = list(dc.index.datasets.search_by_metadata({"properties": {"platform": "landsat-8"}}))
     assert len(lds) == 0
-    lds = list(dc.index.datasets.search_by_metadata({"eo:platform": "landsat-8"}))
+    lds = list(dc.index.datasets.search_by_metadata({"properties": {"eo:platform": "landsat-8"}}))
     assert len(lds) == 2
 
 
@@ -568,3 +576,22 @@ def test_mem_transactions(mem_index_fresh):
     trans.rollback()
     assert not trans.active
     assert mem_index_fresh.index.thread_transaction() is None
+
+
+def test_default_clone_bulk_ops(mem_index_fresh, index, extended_eo3_metadata_type,
+                                ls8_eo3_product, wo_eo3_product, africa_s2_eo3_product,
+                                ls8_eo3_dataset, ls8_eo3_dataset2,
+                                ls8_eo3_dataset3, ls8_eo3_dataset4,
+                                wo_eo3_dataset, africa_eo3_dataset):
+    mem_index_fresh.index.clone(index)
+    assert mem_index_fresh.index.datasets.has(africa_eo3_dataset.id)
+    assert mem_index_fresh.index.datasets.has(wo_eo3_dataset.id)
+    assert mem_index_fresh.index.datasets.has(ls8_eo3_dataset.id)
+    assert mem_index_fresh.index.datasets.has(ls8_eo3_dataset4.id)
+
+
+def test_default_clone_bulk_ops_reverse(mem_eo3_data, index):
+    mem_idx, ls8id, woid = mem_eo3_data
+    index.clone(mem_idx.index)
+    assert index.datasets.has(ls8id)
+    assert index.datasets.has(woid)
