@@ -1,15 +1,7 @@
+# This file is part of the Open Data Cube, see https://opendatacube.org for more information
 #
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+# Copyright (c) 2015-2020 ODC Contributors
+# SPDX-License-Identifier: Apache-2.0
 import datetime
 import pandas
 import numpy as np
@@ -17,23 +9,19 @@ from types import SimpleNamespace
 
 import pytest
 
-from datacube.api.query import Query, _datetime_to_timestamp, query_group_by, solar_day, GroupBy, solar_offset
+from datacube.api.query import Query, query_group_by, solar_day, GroupBy, solar_offset
 from datacube.model import Range
 from datacube.utils import parse_time
 from datacube.utils.geometry import CRS
 
 
-def test_datetime_to_timestamp():
-    assert _datetime_to_timestamp((1990, 1, 7)) == 631670400
-    assert _datetime_to_timestamp(datetime.datetime(1990, 1, 7)) == 631670400
-    assert _datetime_to_timestamp(631670400) == 631670400
-    assert _datetime_to_timestamp('1990-01-07T00:00:00.0Z') == 631670400
+@pytest.fixture
+def mock_index():
+    from unittest.mock import MagicMock
+    return MagicMock()
 
 
-def test_query_kwargs():
-    from mock import MagicMock
-
-    mock_index = MagicMock()
+def test_query_kwargs(mock_index):
     mock_index.datasets.get_field_names = lambda: {u'product', u'lat', u'sat_path', 'type_id', u'time', u'lon',
                                                    u'orbit', u'instrument', u'sat_row', u'platform', 'metadata_type',
                                                    u'gsi', 'type', 'id'}
@@ -79,6 +67,12 @@ def test_query_kwargs():
     query = Query(index=mock_index, time=('2001', '2002'))
     assert 'time' in query.search
 
+    with pytest.raises(ValueError):
+        query = Query(index=mock_index, time=('2001', '2002', '2003'))
+
+    with pytest.raises(ValueError):
+        query = Query(index=mock_index, time=['2001', '2002', '2003'])
+
     with pytest.raises(ValueError):
         Query(index=mock_index,
               y=-4174726, coordinate_reference_system='WGS84',
@@ -143,7 +137,7 @@ testdata = [
      format_test('2008-11-10T11:00', '2008-11-16T14:01:59.999999')),
     ((datetime.date(1995, 1, 1), datetime.date(1999, 1, 1)),
      format_test('1995-01-01T00:00:00', '1999-01-01T23:59:59.999999')),
-    ((datetime.datetime(2008, 1, 1), datetime.date(2008, 1, 4), datetime.datetime(2008, 1, 10, 23, 59, 40)),
+    ((datetime.datetime(2008, 1, 1), datetime.datetime(2008, 1, 10, 23, 59, 40)),
      format_test('2008-01-01T00:00:00', '2008-01-10T23:59:40.999999')),
     ((datetime.date(2008, 1, 1)),
      format_test('2008-01-01T00:00:00', '2008-01-01T23:59:59.999999'))
@@ -210,3 +204,13 @@ def test_dateline_query_building():
                 crs='EPSG:32660').search_terms['lon']
 
     assert lon.begin < 180 < lon.end
+
+
+def test_query_issue_1146():
+    q = Query(k='AB')
+    assert q.search['k'] == 'AB'
+
+
+def test_query_multiple_products(mock_index):
+    q = Query(index=mock_index, product=['ls5_nbar_albers', 'ls7_nbar_albers'])
+    assert q.product == ['ls5_nbar_albers', 'ls7_nbar_albers']
