@@ -9,9 +9,10 @@ from datacube.utils.dates import (
     parse_duration,
     parse_interval,
     parse_time,
-    _parse_time_generic,
     mk_time_coord,
     normalise_dt,
+    tz_aware,
+    tzutc
 )
 
 from dateutil.rrule import YEARLY, MONTHLY, DAILY
@@ -33,9 +34,8 @@ def test_parse():
     with pytest.raises(ValueError):
         parse_interval('1g')
 
-    assert _parse_time_generic('2020-01-01') == parse_time('2020-01-01')
     date = datetime(2020, 2, 3)
-    assert _parse_time_generic(date) is date
+    assert parse_time(date) is date
 
     # test fallback to python parser
     assert parse_time("3 Jan 2020") == datetime(2020, 1, 3)
@@ -51,6 +51,17 @@ def test_normalise_dt():
     assert normalise_dt('2020-03-26T10:15:32.556793+9:00') == datetime(2020, 3, 26, 1, 15, 32, 556793)
 
 
+def test_tz_aware():
+    dt_tz = parse_time('2020-11-15T15:11:56.23456+9:00')
+    assert dt_tz.tzinfo is not None
+    assert tz_aware(dt_tz) is dt_tz
+
+    dt_notz = parse_time('2020-11-15T15:11:56.23456')
+    assert dt_notz.tzinfo is None
+    assert tz_aware(parse_time('2020-11-15T15:11:56.23456')).tzinfo == tzutc()
+    assert tz_aware(parse_time('2020-11-15T15:11:56.23456'), default=dt_tz.tzinfo).tzinfo == dt_tz.tzinfo
+
+
 def test_mk_time_coord():
     t = mk_time_coord(['2020-01-20'])
     assert t.shape == (1,)
